@@ -1,9 +1,9 @@
 package BenchmarkAnything::Storage::Backend::SQL;
-# git description: v0.021-3-gdfc449c
+# git description: v0.022-6-gbc1773c
 
 our $AUTHORITY = 'cpan:TAPPER';
 # ABSTRACT: Autonomous SQL backend to store benchmarks
-$BenchmarkAnything::Storage::Backend::SQL::VERSION = '0.022';
+$BenchmarkAnything::Storage::Backend::SQL::VERSION = '0.023';
 use 5.008;
 use utf8;
 use strict;
@@ -34,9 +34,6 @@ my $hr_column_ba_mapping = {
     created_at     => 'CREATED',
 };
 
-sub json_true  { bless( do{\(my $o = 1)}, 'JSON::PP::Boolean' ) }
-sub json_false { bless( do{\(my $o = 0)}, 'JSON::PP::Boolean' ) }
-
 my $fn_add_subsumed_point = sub {
 
     my ( $or_self, $hr_atts ) = @_;
@@ -160,12 +157,15 @@ sub new {
             dbh    => $hr_atts->{dbh},
             driver => $hr_atts->{dbh}{Driver}{Name},
             debug  => $hr_atts->{debug} || 0,
+            verbose=> $hr_atts->{verbose} || 0,
             config => $or_self->{config},
         });
     }
 
     $or_self->{searchengine} = $hr_atts->{searchengine} if $hr_atts->{searchengine};
     $or_self->{debug}        = $hr_atts->{debug} || 0;
+    $or_self->{verbose}      = $hr_atts->{verbose} || 0;
+    $or_self->{dbh_config}   = $hr_atts->{dbh_config};
 
     return $or_self;
 
@@ -604,6 +604,10 @@ sub get_stats {
                  index_single_added_values_immediately => $or_self->{searchengine}{elasticsearch}{index_single_added_values_immediately} || 0,
              },
             );
+        # boolean -> 0/1
+        for (values %{$h_searchengine_stats{elasticsearch}{cluster_health}}) {
+            $_ = $_ ? 1 : 0 if ref eq 'JSON::XS::Boolean';
+        }
     }
 
     return {
@@ -779,12 +783,13 @@ sub search_array {
             {
                 if ($field_mapping->{$sort_field}{type} and $field_mapping->{$sort_field}{type} eq 'text')
                 {
+                    require BenchmarkAnything::Storage::Backend::SQL::Search;
                     $or_es->indices->put_mapping
                      (
                       index => $s_index,
                       type => $s_type,
                       body => { $s_type => { properties => { $sort_field => { type => 'text',
-                                                                              fielddata => json_true,
+                                                                              fielddata => BenchmarkAnything::Storage::Backend::SQL::Search::json_true(),
                                                                             }}}}
                      );
                 }
@@ -1007,99 +1012,14 @@ sub init_search_engine
 {
     my ( $or_self, $b_force) = @_;
 
-    my $debug = $or_self->{debug} || $or_self->{searchengine}{elasticsearch}{debug};
-
-    if ( $or_self->{searchengine}{elasticsearch}{index} )
-    {
-        require BenchmarkAnything::Storage::Search::Elasticsearch;
-        my ($or_es, $s_index, $s_type) = BenchmarkAnything::Storage::Search::Elasticsearch::get_elasticsearch_client
-         (
-          {searchengine => $or_self->{searchengine}}
-         );
-
-        # exists?
-        if ($or_es->indices->exists(index => $s_index) and not $b_force)
-        {
-            print STDERR "init_search_engine: index '$s_index' already exists, use force to delete and recreate.\n";
-            return;
-        }
-
-        # delete
-        if ($or_es->indices->exists(index => $s_index))
-        {
-            my $response = $or_es->indices->delete(index => $s_index);
-        }
-
-        # mappings
-        my $mappings =
-        {
-         $s_type => {
-          dynamic_templates =>
-          [
-           { "core_field_NAME"     => { "match" => "NAME",     "mapping" => { "type" => "keyword", "store" => json_true } } },
-           { "core_field_VALUE"    => { "match" => "VALUE",    "mapping" => { "type" => "text", } } },
-           { "core_field_UNIT"     => { "match" => "UNIT",     "mapping" => { "type" => "text", } } },
-           { "core_field_VALUE_ID" => { "match" => "VALUE_ID", "mapping" => { "type" => "long", } } },
-           { "core_field_CREATED"  => { "match" => "CREATED",  "mapping" => { "type" => "date", format => 'yyyy-MM-dd||yyyy-MM-dd HH:mm:ss', } } },
-           { "non_core_fields"     => { "match" => "*",        "mapping" => { "type" => "keyword", } } },
-          ]
-         }
-        };
-        require JSON::XS;
-        require Hash::Merge;
-        require Data::Dumper;
-        local $Types::Serialiser::true;
-        my $merge = Hash::Merge->new( 'RIGHT_PRECEDENT' );
-        my $additional_mappings = $or_self->{searchengine}{elasticsearch}{additional_mappings} || {};
-        $mappings = $merge->merge($mappings, $additional_mappings);
-        print STDERR "create.mappings:\n".JSON::XS->new->convert_blessed->pretty->encode($mappings) if $debug;
-
-        # create
-        my $answer = $or_es->indices->create
-         (
-          index => $s_index,
-          body  => { mappings => $mappings },
-         );
-        print STDERR "create.answer:\n".Data::Dumper::Dumper($answer) if $debug;
-    }
+    require BenchmarkAnything::Storage::Backend::SQL::Search;
+    BenchmarkAnything::Storage::Backend::SQL::Search::init_search_engine (@_);
 }
 
 sub sync_search_engine
 {
-    my ( $or_self, $b_force, $i_start, $i_count) = @_;
-
-    $i_start ||= 1;
-    $i_count ||= 10_000;
-
-    if ($or_self->{searchengine}{elasticsearch})
-    {
-        require BenchmarkAnything::Storage::Search::Elasticsearch;
-        my ($or_es, $s_index, $s_type) = BenchmarkAnything::Storage::Search::Elasticsearch::get_elasticsearch_client
-         (
-          {searchengine => $or_self->{searchengine}, ownjson => 1}
-         );
-        my $bulk = $or_es->bulk_helper(index => $s_index, type => $s_type);
-        my $i_count_datapoints = $or_self->{query}->select_count_datapoints->fetch->[0];
-
-        for (my $i = $i_start; $i <= $i_count_datapoints; $i += $i_count)
-        {
-            if ($b_force
-                # If the beginning or the end of a range do not exist we sync that window.
-                # Careful! This will not sync that window if there is a holein between! So
-                # if in doubt you better force a full sync.
-                or not $or_es->exists(index => $s_index, type => $s_type, id => $i)
-                or not $or_es->exists(index => $s_index, type => $s_type, id => $i+$i_count-1)
-                )
-            {
-                # Make sure to query the ::Backend::SQL store!
-                my $bmks = $or_self->get_full_benchmark_points($i, $i_count);
-                $bulk->index({ id => $_->{VALUE_ID}, source => $_}) foreach @$bmks;
-                $bulk->flush;
-            }
-        }
-    } else {
-            # Unsupported search engine
-    }
+    require BenchmarkAnything::Storage::Backend::SQL::Search;
+    BenchmarkAnything::Storage::Backend::SQL::Search::sync_search_engine (@_);
 }
 
 1;
@@ -1209,6 +1129,7 @@ Create a new B<BenchmarkAnything::Storage::Backend::SQL> object.
     my $or_bench = BenchmarkAnything::Storage::Backend::SQL->new({
         dbh    => $or_dbh,
         debug  => 0,
+        verbose=> 0,
         config => YAML::Syck::LoadFile('~/conf/tapper_benchmark.conf'),
         searchengine => ... # optional, see below at "Elasticsearch support"
     });
@@ -1229,6 +1150,11 @@ B<Configuration> for details.
 Setting C<debug> to a true value results in multiple debugging informations
 written to STDOUT. The default is 0.
 
+=item verbose [optional]
+
+Setting C<verbose> to a true value provides more logs or status
+information. The default is 0.
+
 =back
 
 =head3 add_single_benchmark
@@ -1707,8 +1633,8 @@ Containing the names of the tables used bei B<BenchmarkAnything::Storage::Backen
         unit_table                       => 'bench_units',
         benchmark_table                  => 'benchs',
         benchmark_value_table            => 'bench_values',
-        benchmark_backup_value_table     => 'bench_backup_values',
         subsume_type_table               => 'bench_subsume_types',
+        benchmark_backup_value_table     => 'bench_backup_values',
         additional_type_table            => 'bench_additional_types',
         additional_value_table           => 'bench_additional_values',
         additional_relation_table        => 'bench_additional_relations',
